@@ -18,17 +18,12 @@ package certs
 
 import (
 	"bytes"
-	"crypto"
-	"crypto/sha256"
 	"crypto/x509"
 	"net"
 	"os"
 	"path/filepath"
 	"testing"
 
-	"github.com/pkg/errors"
-	"github.com/stretchr/testify/assert"
-
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
 	certutil "k8s.io/client-go/util/cert"
 	"k8s.io/client-go/util/keyutil"
@@ -36,25 +31,12 @@ import (
 	kubeadmapi "k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm"
 	kubeadmconstants "k8s.io/kubernetes/cmd/kubeadm/app/constants"
 	certstestutil "k8s.io/kubernetes/cmd/kubeadm/app/util/certs"
+	"k8s.io/kubernetes/cmd/kubeadm/app/util/errors"
 	"k8s.io/kubernetes/cmd/kubeadm/app/util/pkiutil"
 	pkiutiltesting "k8s.io/kubernetes/cmd/kubeadm/app/util/pkiutil/testing"
 	testutil "k8s.io/kubernetes/cmd/kubeadm/test"
 )
 
-func createTestCSR(t *testing.T) (*x509.CertificateRequest, crypto.Signer) {
-	csr, key, err := pkiutil.NewCSRAndKey(
-		&pkiutil.CertConfig{
-			Config: certutil.Config{
-				CommonName: "testCert",
-			},
-		})
-	if err != nil {
-		t.Fatalf("couldn't create test cert: %v", err)
-	}
-
-	return csr, key
-}
-
 func TestWriteCertificateAuthorityFilesIfNotExist(t *testing.T) {
 	setupCert, setupKey := certstestutil.CreateCACert(t)
 	caCert, caKey := certstestutil.CreateCACert(t)
@@ -91,8 +73,7 @@ func TestWriteCertificateAuthorityFilesIfNotExist(t *testing.T) {
 
 	for _, test := range tests {
 		// Create temp folder for the test case
-		tmpdir := testutil.SetupTempDir(t)
-		defer os.RemoveAll(tmpdir)
+		tmpdir := t.TempDir()
 
 		// executes setup func (if necessary)
 		if test.setupFunc != nil {
@@ -196,8 +177,7 @@ func TestWriteCertificateFilesIfNotExist(t *testing.T) {
 
 	for _, test := range tests {
 		// Create temp folder for the test case
-		tmpdir := testutil.SetupTempDir(t)
-		defer os.RemoveAll(tmpdir)
+		tmpdir := t.TempDir()
 
 		// executes setup func (if necessary)
 		if test.setupFunc != nil {
@@ -235,75 +215,6 @@ func TestWriteCertificateFilesIfNotExist(t *testing.T) {
 	}
 }
 
-func TestWriteCSRFilesIfNotExist(t *testing.T) {
-	csr, key := createTestCSR(t)
-	csr2, key2 := createTestCSR(t)
-
-	var tests = []struct {
-		name          string
-		setupFunc     func(csrPath string) error
-		expectedError bool
-		expectedCSR   *x509.CertificateRequest
-	}{
-		{
-			name:        "no files exist",
-			expectedCSR: csr,
-		},
-		{
-			name: "other key exists",
-			setupFunc: func(csrPath string) error {
-				if err := pkiutil.WriteCSR(csrPath, "dummy", csr2); err != nil {
-					return err
-				}
-				return pkiutil.WriteKey(csrPath, "dummy", key2)
-			},
-			expectedCSR: csr2,
-		},
-		{
-			name: "existing CSR is garbage",
-			setupFunc: func(csrPath string) error {
-				return os.WriteFile(filepath.Join(csrPath, "dummy.csr"), []byte("a--bunch--of-garbage"), os.ModePerm)
-			},
-			expectedError: true,
-		},
-	}
-
-	for _, test := range tests {
-		t.Run(test.name, func(t *testing.T) {
-			tmpdir := testutil.SetupTempDir(t)
-			defer os.RemoveAll(tmpdir)
-
-			if test.setupFunc != nil {
-				if err := test.setupFunc(tmpdir); err != nil {
-					t.Fatalf("couldn't set up test: %v", err)
-				}
-			}
-
-			if err := writeCSRFilesIfNotExist(tmpdir, "dummy", csr, key); err != nil {
-				if test.expectedError {
-					return
-				}
-				t.Fatalf("unexpected error %v: ", err)
-			}
-
-			if test.expectedError {
-				t.Fatal("Expected error, but got none")
-			}
-
-			parsedCSR, _, err := pkiutil.TryLoadCSRAndKeyFromDisk(tmpdir, "dummy")
-			if err != nil {
-				t.Fatalf("couldn't load csr and key: %v", err)
-			}
-
-			if sha256.Sum256(test.expectedCSR.Raw) != sha256.Sum256(parsedCSR.Raw) {
-				t.Error("expected csr's fingerprint does not match ")
-			}
-
-		})
-	}
-
-}
-
 func TestCreateServiceAccountKeyAndPublicKeyFiles(t *testing.T) {
 	setupKey, err := keyutil.MakeEllipticPrivateKeyPEM()
 	if err != nil {
@@ -338,8 +249,7 @@ func TestCreateServiceAccountKeyAndPublicKeyFiles(t *testing.T) {
 	}
 	for _, tt := range tcases {
 		t.Run(tt.name, func(t *testing.T) {
-			dir := testutil.SetupTempDir(t)
-			defer os.RemoveAll(dir)
+			dir := t.TempDir()
 
 			if tt.setupFunc != nil {
 				if err := tt.setupFunc(dir); err != nil {
@@ -347,7 +257,7 @@ func TestCreateServiceAccountKeyAndPublicKeyFiles(t *testing.T) {
 				}
 			}
 
-			err := CreateServiceAccountKeyAndPublicKeyFiles(dir, kubeadmapi.EncryptionAlgorithmRSA)
+			err := CreateServiceAccountKeyAndPublicKeyFiles(dir, kubeadmapi.EncryptionAlgorithmRSA2048)
 			if (err != nil) != tt.expectedErr {
 				t.Fatalf("expected error: %v, got: %v, error: %v", tt.expectedErr, err != nil, err)
 			} else if tt.expectedErr {
@@ -469,9 +379,8 @@ func TestSharedCertificateExists(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run("", func(t *testing.T) {
-			tmpdir := testutil.SetupTempDir(t)
+			tmpdir := t.TempDir()
 			os.MkdirAll(tmpdir+"/etcd", os.ModePerm)
-			defer os.RemoveAll(tmpdir)
 
 			cfg := &kubeadmapi.ClusterConfiguration{
 				CertificatesDir: tmpdir,
@@ -501,8 +410,7 @@ func TestCreatePKIAssetsWithSparseCerts(t *testing.T) {
 		t.Run(test.Name, func(t *testing.T) {
 			pkiutiltesting.Reset()
 
-			tmpdir := testutil.SetupTempDir(t)
-			defer os.RemoveAll(tmpdir)
+			tmpdir := t.TempDir()
 
 			cfg := testutil.GetDefaultInternalConfig(t)
 			cfg.ClusterConfiguration.CertificatesDir = tmpdir
@@ -604,8 +512,7 @@ func TestUsingExternalCA(t *testing.T) {
 		t.Run(test.name, func(t *testing.T) {
 			pkiutiltesting.Reset()
 
-			dir := testutil.SetupTempDir(t)
-			defer os.RemoveAll(dir)
+			dir := t.TempDir()
 
 			cfg := &kubeadmapi.InitConfiguration{
 				LocalAPIEndpoint: kubeadmapi.APIEndpoint{AdvertiseAddress: "1.2.3.4"},
@@ -708,8 +615,7 @@ func TestValidateMethods(t *testing.T) {
 	}
 
 	for _, test := range tests {
-		dir := testutil.SetupTempDir(t)
-		defer os.RemoveAll(dir)
+		dir := t.TempDir()
 		test.loc.pkiDir = dir
 
 		certstestutil.WritePKIFiles(t, dir, test.files)
@@ -723,38 +629,6 @@ func TestValidateMethods(t *testing.T) {
 	}
 }
 
-func TestNewCSR(t *testing.T) {
-	kubeadmCert := KubeadmCertAPIServer()
-	cfg := testutil.GetDefaultInternalConfig(t)
-
-	certConfig, err := kubeadmCert.GetConfig(cfg)
-	if err != nil {
-		t.Fatalf("couldn't get cert config: %v", err)
-	}
-
-	csr, _, err := NewCSR(kubeadmCert, cfg)
-
-	if err != nil {
-		t.Errorf("invalid signature on CSR: %v", err)
-	}
-
-	assert.ElementsMatch(t, certConfig.Organization, csr.Subject.Organization, "organizations not equal")
-
-	if csr.Subject.CommonName != certConfig.CommonName {
-		t.Errorf("expected common name %q, got %q", certConfig.CommonName, csr.Subject.CommonName)
-	}
-
-	assert.ElementsMatch(t, certConfig.AltNames.DNSNames, csr.DNSNames, "dns names not equal")
-
-	assert.Len(t, csr.IPAddresses, len(certConfig.AltNames.IPs))
-
-	for i, ip := range csr.IPAddresses {
-		if !ip.Equal(certConfig.AltNames.IPs[i]) {
-			t.Errorf("[%d]: %v != %v", i, ip, certConfig.AltNames.IPs[i])
-		}
-	}
-}
-
 func TestCreateCertificateFilesMethods(t *testing.T) {
 
 	var tests = []struct {
@@ -796,8 +670,7 @@ func TestCreateCertificateFilesMethods(t *testing.T) {
 		pkiutiltesting.Reset()
 
 		// Create temp folder for the test case
-		tmpdir := testutil.SetupTempDir(t)
-		defer os.RemoveAll(tmpdir)
+		tmpdir := t.TempDir()
 
 		cfg := &kubeadmapi.InitConfiguration{
 			LocalAPIEndpoint: kubeadmapi.APIEndpoint{AdvertiseAddress: "1.2.3.4"},
