@@ -22,19 +22,19 @@ import (
 	"reflect"
 	"strconv"
 	"testing"
-
-	"github.com/pkg/errors"
+	"time"
 
 	pb "go.etcd.io/etcd/api/v3/etcdserverpb"
 	clientv3 "go.etcd.io/etcd/client/v3"
+
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/runtime"
-	"k8s.io/apimachinery/pkg/util/wait"
 	clientsetfake "k8s.io/client-go/kubernetes/fake"
 	clienttesting "k8s.io/client-go/testing"
 
 	kubeadmapi "k8s.io/kubernetes/cmd/kubeadm/app/apis/kubeadm"
 	"k8s.io/kubernetes/cmd/kubeadm/app/constants"
+	"k8s.io/kubernetes/cmd/kubeadm/app/util/errors"
 	testresources "k8s.io/kubernetes/cmd/kubeadm/test/resources"
 )
 
@@ -57,7 +57,7 @@ func (f *fakeEtcdClient) Endpoints() []string {
 }
 
 // MemberList lists the current cluster membership.
-func (f *fakeEtcdClient) MemberList(_ context.Context) (*clientv3.MemberListResponse, error) {
+func (f *fakeEtcdClient) MemberList(_ context.Context, _ ...clientv3.OpOption) (*clientv3.MemberListResponse, error) {
 	return &clientv3.MemberListResponse{
 		Members: f.members,
 	}, nil
@@ -135,7 +135,7 @@ func TestGetClientURL(t *testing.T) {
 }
 
 func TestGetPeerURL(t *testing.T) {
-	testGetURL(t, GetClientURL, constants.EtcdListenClientPort)
+	testGetURL(t, GetPeerURL, constants.EtcdListenPeerPort)
 }
 
 func TestGetClientURLByIP(t *testing.T) {
@@ -208,7 +208,7 @@ func TestGetEtcdEndpointsWithBackoff(t *testing.T) {
 					t.Errorf("error setting up test creating pod for node %q", pod.NodeName)
 				}
 			}
-			endpoints, err := getEtcdEndpointsWithBackoff(client, wait.Backoff{Duration: 0, Jitter: 0, Steps: 1})
+			endpoints, err := getEtcdEndpointsWithRetry(client, time.Microsecond*10, time.Millisecond*100)
 			if err != nil && !rt.expectedErr {
 				t.Errorf("got error %q; was expecting no errors", err)
 				return
@@ -293,7 +293,7 @@ func TestGetRawEtcdEndpointsFromPodAnnotation(t *testing.T) {
 			if rt.clientSetup != nil {
 				rt.clientSetup(client)
 			}
-			endpoints, err := getRawEtcdEndpointsFromPodAnnotation(client, wait.Backoff{Duration: 0, Jitter: 0, Steps: 1})
+			endpoints, err := getRawEtcdEndpointsFromPodAnnotation(client, time.Microsecond*10, time.Millisecond*100)
 			if err != nil && !rt.expectedErr {
 				t.Errorf("got error %v, but wasn't expecting any error", err)
 				return
@@ -482,6 +482,11 @@ func TestClient_GetMemberID(t *testing.T) {
 				Endpoints:     tt.fields.Endpoints,
 				newEtcdClient: tt.fields.newEtcdClient,
 			}
+			c.listMembersFunc = func(_ time.Duration) (*clientv3.MemberListResponse, error) {
+				f, _ := c.newEtcdClient([]string{})
+				resp, _ := f.MemberList(context.Background())
+				return resp, nil
+			}
 
 			got, err := c.GetMemberID(tt.args.peerURL)
 			if !errors.Is(tt.wantErr, err) {
@@ -494,3 +499,292 @@ func TestClient_GetMemberID(t *testing.T) {
 		})
 	}
 }
+
+func TestListMembers(t *testing.T) {
+	type fields struct {
+		Endpoints       []string
+		newEtcdClient   func(endpoints []string) (etcdClient, error)
+		listMembersFunc func(timeout time.Duration) (*clientv3.MemberListResponse, error)
+	}
+	tests := []struct {
+		name      string
+		fields    fields
+		want      []Member
+		wantError bool
+	}{
+		{
+			name: "PeerURLs are empty",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{}
+					return f, nil
+				},
+			},
+			want: []Member{},
+		},
+		{
+			name: "PeerURLs are non-empty",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{
+							{
+								ID:   1,
+								Name: "member1",
+								PeerURLs: []string{
+									"https://member1:2380",
+								},
+							},
+							{
+								ID:   2,
+								Name: "member2",
+								PeerURLs: []string{
+									"https://member2:2380",
+								},
+							},
+						},
+					}
+					return f, nil
+				},
+			},
+			want: []Member{
+				{
+					Name:    "member1",
+					PeerURL: "https://member1:2380",
+				},
+				{
+					Name:    "member2",
+					PeerURL: "https://member2:2380",
+				},
+			},
+		},
+		{
+			name: "PeerURLs has multiple urls",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{
+							{
+								ID:   1,
+								Name: "member1",
+								PeerURLs: []string{
+									"https://member1:2380",
+									"https://member2:2380",
+								},
+							},
+						},
+					}
+					return f, nil
+				},
+			},
+			want: []Member{
+				{
+					Name:    "member1",
+					PeerURL: "https://member1:2380",
+				},
+			},
+		},
+		{
+			name: "ListMembers return error",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{
+							{
+								ID:   1,
+								Name: "member1",
+								PeerURLs: []string{
+									"https://member1:2380",
+									"https://member2:2380",
+								},
+							},
+						},
+					}
+					return f, nil
+				},
+				listMembersFunc: func(_ time.Duration) (*clientv3.MemberListResponse, error) {
+					return nil, errNotImplemented
+				},
+			},
+			want:      nil,
+			wantError: true,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			c := &Client{
+				Endpoints:       tt.fields.Endpoints,
+				newEtcdClient:   tt.fields.newEtcdClient,
+				listMembersFunc: tt.fields.listMembersFunc,
+			}
+			if c.listMembersFunc == nil {
+				c.listMembersFunc = func(_ time.Duration) (*clientv3.MemberListResponse, error) {
+					return c.listMembers(100 * time.Millisecond)
+				}
+			}
+			got, err := c.ListMembers()
+			if !reflect.DeepEqual(got, tt.want) {
+				t.Errorf("ListMembers() = %v, want %v", got, tt.want)
+			}
+			if (err != nil) != (tt.wantError) {
+				t.Errorf("ListMembers() error = %v, wantError %v", err, tt.wantError)
+			}
+		})
+	}
+}
+
+func TestIsLearner(t *testing.T) {
+	type fields struct {
+		Endpoints       []string
+		newEtcdClient   func(endpoints []string) (etcdClient, error)
+		listMembersFunc func(timeout time.Duration) (*clientv3.MemberListResponse, error)
+	}
+	tests := []struct {
+		name      string
+		fields    fields
+		memberID  uint64
+		want      bool
+		wantError bool
+	}{
+		{
+			name: "The specified member is not a learner",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{
+							{
+								ID:   1,
+								Name: "member1",
+								PeerURLs: []string{
+									"https://member1:2380",
+								},
+								IsLearner: false,
+							},
+						},
+					}
+					return f, nil
+				},
+			},
+			memberID: 1,
+			want:     false,
+		},
+		{
+			name: "The specified member is a learner",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{
+							{
+								ID:   1,
+								Name: "member1",
+								PeerURLs: []string{
+									"https://member1:2380",
+								},
+								IsLearner: true,
+							},
+							{
+								ID:   2,
+								Name: "member2",
+								PeerURLs: []string{
+									"https://member2:2380",
+								},
+							},
+						},
+					}
+					return f, nil
+				},
+			},
+			memberID: 1,
+			want:     true,
+		},
+		{
+			name: "The specified member does not exist",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{},
+					}
+					return f, nil
+				},
+			},
+			memberID: 3,
+			want:     false,
+		},
+		{
+			name: "Learner ID is empty",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{
+							{
+								Name: "member2",
+								PeerURLs: []string{
+									"https://member2:2380",
+								},
+								IsLearner: true,
+							},
+						},
+					}
+					return f, nil
+				},
+			},
+			want: true,
+		},
+		{
+			name: "ListMembers returns an error",
+			fields: fields{
+				Endpoints: []string{},
+				newEtcdClient: func(endpoints []string) (etcdClient, error) {
+					f := &fakeEtcdClient{
+						members: []*pb.Member{
+							{
+								Name: "member2",
+								PeerURLs: []string{
+									"https://member2:2380",
+								},
+								IsLearner: true,
+							},
+						},
+					}
+					return f, nil
+				},
+				listMembersFunc: func(_ time.Duration) (*clientv3.MemberListResponse, error) {
+					return nil, errNotImplemented
+				},
+			},
+			want:      false,
+			wantError: true,
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			c := &Client{
+				Endpoints:       tt.fields.Endpoints,
+				newEtcdClient:   tt.fields.newEtcdClient,
+				listMembersFunc: tt.fields.listMembersFunc,
+			}
+			if c.listMembersFunc == nil {
+				c.listMembersFunc = func(t_ time.Duration) (*clientv3.MemberListResponse, error) {
+					f, _ := c.newEtcdClient([]string{})
+					resp, _ := f.MemberList(context.Background())
+					return resp, nil
+				}
+			}
+			got, err := c.isLearner(tt.memberID)
+			if got != tt.want {
+				t.Errorf("isLearner() = %v, want %v", got, tt.want)
+			}
+			if (err != nil) != (tt.wantError) {
+				t.Errorf("isLearner() error = %v, wantError %v", err, tt.wantError)
+			}
+		})
+	}
+}
