@@ -17,28 +17,34 @@ limitations under the License.
 package cpumanager
 
 import (
+	"context"
 	"fmt"
+	"os"
 	"reflect"
+	"runtime"
 	"strconv"
 	"strings"
 	"testing"
 	"time"
 
-	"io/ioutil"
-	"os"
+	utilfeature "k8s.io/apiserver/pkg/util/feature"
+	featuregatetesting "k8s.io/component-base/featuregate/testing"
+	"k8s.io/kubernetes/pkg/features"
 
+	"github.com/go-logr/logr"
 	cadvisorapi "github.com/google/cadvisor/info/v1"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	runtimeapi "k8s.io/cri-api/pkg/apis/runtime/v1"
+	"k8s.io/kubernetes/test/utils/ktesting"
 
 	"k8s.io/kubernetes/pkg/kubelet/cm/containermap"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpumanager/state"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpumanager/topology"
-	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
 	"k8s.io/kubernetes/pkg/kubelet/cm/topologymanager"
+	"k8s.io/utils/cpuset"
 )
 
 type mockState struct {
@@ -101,35 +107,35 @@ func (p *mockPolicy) Name() string {
 	return "mock"
 }
 
-func (p *mockPolicy) Start(s state.State) error {
+func (p *mockPolicy) Start(_ logr.Logger, s state.State) error {
 	return p.err
 }
 
-func (p *mockPolicy) Allocate(s state.State, pod *v1.Pod, container *v1.Container) error {
+func (p *mockPolicy) Allocate(_ logr.Logger, s state.State, pod *v1.Pod, container *v1.Container) error {
 	return p.err
 }
 
-func (p *mockPolicy) RemoveContainer(s state.State, podUID string, containerName string) error {
+func (p *mockPolicy) RemoveContainer(_ logr.Logger, s state.State, podUID string, containerName string) error {
 	return p.err
 }
 
-func (p *mockPolicy) GetTopologyHints(s state.State, pod *v1.Pod, container *v1.Container) map[string][]topologymanager.TopologyHint {
+func (p *mockPolicy) GetTopologyHints(_ logr.Logger, s state.State, pod *v1.Pod, container *v1.Container) map[string][]topologymanager.TopologyHint {
 	return nil
 }
 
-func (p *mockPolicy) GetPodTopologyHints(s state.State, pod *v1.Pod) map[string][]topologymanager.TopologyHint {
+func (p *mockPolicy) GetPodTopologyHints(_ logr.Logger, s state.State, pod *v1.Pod) map[string][]topologymanager.TopologyHint {
 	return nil
 }
 
 func (p *mockPolicy) GetAllocatableCPUs(m state.State) cpuset.CPUSet {
-	return cpuset.NewCPUSet()
+	return cpuset.New()
 }
 
 type mockRuntimeService struct {
 	err error
 }
 
-func (rt mockRuntimeService) UpdateContainerResources(id string, resources *runtimeapi.LinuxContainerResources) error {
+func (rt mockRuntimeService) UpdateContainerResources(_ context.Context, id string, resources *runtimeapi.ContainerResources) error {
 	return rt.err
 }
 
@@ -168,6 +174,22 @@ func makePod(podUID, containerName, cpuRequest, cpuLimit string) *v1.Pod {
 	return pod
 }
 
+func makePodWithPodLevelResources(podUID, podCPURequest, podCPULimit, containerName, cpuRequest, cpuLimit string) *v1.Pod {
+	pod := makePod(podUID, containerName, cpuRequest, cpuLimit)
+	pod.Spec.Resources = &v1.ResourceRequirements{
+		Requests: v1.ResourceList{
+			v1.ResourceName(v1.ResourceCPU):    resource.MustParse(podCPURequest),
+			v1.ResourceName(v1.ResourceMemory): resource.MustParse("1G"),
+		},
+		Limits: v1.ResourceList{
+			v1.ResourceName(v1.ResourceCPU):    resource.MustParse(podCPULimit),
+			v1.ResourceName(v1.ResourceMemory): resource.MustParse("1G"),
+		},
+	}
+
+	return pod
+}
+
 func makeMultiContainerPod(initCPUs, appCPUs []struct{ request, limit string }) *v1.Pod {
 	pod := &v1.Pod{
 		ObjectMeta: metav1.ObjectMeta{
@@ -215,8 +237,63 @@ func makeMultiContainerPod(initCPUs, appCPUs []struct{ request, limit string })
 	return pod
 }
 
+func makeMultiContainerPodWithOptions(initCPUs, appCPUs []*containerOptions) *v1.Pod {
+	pod := &v1.Pod{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "pod",
+			UID:  "podUID",
+		},
+		Spec: v1.PodSpec{
+			InitContainers: []v1.Container{},
+			Containers:     []v1.Container{},
+		},
+	}
+
+	for i, cpu := range initCPUs {
+		pod.Spec.InitContainers = append(pod.Spec.InitContainers, v1.Container{
+			Name: "initContainer-" + strconv.Itoa(i),
+			Resources: v1.ResourceRequirements{
+				Requests: v1.ResourceList{
+					v1.ResourceName(v1.ResourceCPU):    resource.MustParse(cpu.request),
+					v1.ResourceName(v1.ResourceMemory): resource.MustParse("1G"),
+				},
+				Limits: v1.ResourceList{
+					v1.ResourceName(v1.ResourceCPU):    resource.MustParse(cpu.limit),
+					v1.ResourceName(v1.ResourceMemory): resource.MustParse("1G"),
+				},
+			},
+			RestartPolicy: &cpu.restartPolicy,
+		})
+	}
+
+	for i, cpu := range appCPUs {
+		pod.Spec.Containers = append(pod.Spec.Containers, v1.Container{
+			Name: "appContainer-" + strconv.Itoa(i),
+			Resources: v1.ResourceRequirements{
+				Requests: v1.ResourceList{
+					v1.ResourceName(v1.ResourceCPU):    resource.MustParse(cpu.request),
+					v1.ResourceName(v1.ResourceMemory): resource.MustParse("1G"),
+				},
+				Limits: v1.ResourceList{
+					v1.ResourceName(v1.ResourceCPU):    resource.MustParse(cpu.limit),
+					v1.ResourceName(v1.ResourceMemory): resource.MustParse("1G"),
+				},
+			},
+		})
+	}
+
+	return pod
+}
+
 func TestCPUManagerAdd(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
+	logger, _ := ktesting.NewTestContext(t)
+
 	testPolicy, _ := NewStaticPolicy(
+		logger,
 		&topology.CPUTopology{
 			NumCPUs:    4,
 			NumSockets: 1,
@@ -229,7 +306,7 @@ func TestCPUManagerAdd(t *testing.T) {
 			},
 		},
 		0,
-		cpuset.NewCPUSet(),
+		cpuset.New(),
 		topologymanager.NewFakeManager(),
 		nil)
 	testCases := []struct {
@@ -244,7 +321,7 @@ func TestCPUManagerAdd(t *testing.T) {
 			description:        "cpu manager add - no error",
 			updateErr:          nil,
 			policy:             testPolicy,
-			expCPUSet:          cpuset.NewCPUSet(3, 4),
+			expCPUSet:          cpuset.New(3, 4),
 			expAllocateErr:     nil,
 			expAddContainerErr: nil,
 		},
@@ -254,7 +331,7 @@ func TestCPUManagerAdd(t *testing.T) {
 			policy: &mockPolicy{
 				err: fmt.Errorf("fake reg error"),
 			},
-			expCPUSet:          cpuset.NewCPUSet(1, 2, 3, 4),
+			expCPUSet:          cpuset.New(1, 2, 3, 4),
 			expAllocateErr:     fmt.Errorf("fake reg error"),
 			expAddContainerErr: nil,
 		},
@@ -265,9 +342,9 @@ func TestCPUManagerAdd(t *testing.T) {
 			policy: testCase.policy,
 			state: &mockState{
 				assignments:   state.ContainerCPUAssignments{},
-				defaultCPUSet: cpuset.NewCPUSet(1, 2, 3, 4),
+				defaultCPUSet: cpuset.New(1, 2, 3, 4),
 			},
-			lastUpdateState: state.NewMemoryState(),
+			lastUpdateState: state.NewMemoryState(logger),
 			containerRuntime: mockRuntimeService{
 				err: testCase.updateErr,
 			},
@@ -278,7 +355,7 @@ func TestCPUManagerAdd(t *testing.T) {
 
 		pod := makePod("fakePod", "fakeContainer", "2", "2")
 		container := &pod.Spec.Containers[0]
-		mgr.activePods = func() []*v1.Pod { return nil }
+		mgr.activePods = func() []*v1.Pod { return []*v1.Pod{pod} }
 
 		err := mgr.Allocate(pod, container)
 		if !reflect.DeepEqual(err, testCase.expAllocateErr) {
@@ -286,7 +363,7 @@ func TestCPUManagerAdd(t *testing.T) {
 				testCase.description, testCase.expAllocateErr, err)
 		}
 
-		mgr.AddContainer(pod, container, "fakeID")
+		mgr.AddContainer(logger, pod, container, "fakeID")
 		_, _, err = mgr.containerMap.GetContainerRef("fakeID")
 		if !reflect.DeepEqual(err, testCase.expAddContainerErr) {
 			t.Errorf("CPU Manager AddContainer() error (%v). expected error: %v but got: %v",
@@ -300,6 +377,10 @@ func TestCPUManagerAdd(t *testing.T) {
 }
 
 func TestCPUManagerAddWithInitContainers(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
 	testCases := []struct {
 		description      string
 		topo             *topology.CPUTopology
@@ -317,71 +398,71 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
 				[]struct{ request, limit string }{{"100m", "100m"}},
 				[]struct{ request, limit string }{{"4000m", "4000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet()},
+				cpuset.New()},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4, 1, 5)},
 		},
 		{
 			description:      "Equal Number of Guaranteed CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
 				[]struct{ request, limit string }{{"4000m", "4000m"}},
 				[]struct{ request, limit string }{{"4000m", "4000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4, 1, 5)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4, 1, 5)},
 		},
 		{
 			description:      "More Init Container Guaranteed CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
 				[]struct{ request, limit string }{{"6000m", "6000m"}},
 				[]struct{ request, limit string }{{"4000m", "4000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5, 2, 6)},
+				cpuset.New(0, 4, 1, 5, 2, 6)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4, 1, 5)},
 		},
 		{
 			description:      "Less Init Container Guaranteed CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
 				[]struct{ request, limit string }{{"2000m", "2000m"}},
 				[]struct{ request, limit string }{{"4000m", "4000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4)},
+				cpuset.New(0, 4)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4, 1, 5)},
 		},
 		{
 			description:      "Multi Init Container Equal CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID-1", "initFakeID-2"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
@@ -391,17 +472,17 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 				[]struct{ request, limit string }{
 					{"2000m", "2000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4),
-				cpuset.NewCPUSet(0, 4)},
+				cpuset.New(0, 4),
+				cpuset.New(0, 4)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4)},
+				cpuset.New(0, 4)},
 		},
 		{
 			description:      "Multi Init Container Less CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID-1", "initFakeID-2"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
@@ -411,17 +492,17 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 				[]struct{ request, limit string }{
 					{"2000m", "2000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5),
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4, 1, 5),
+				cpuset.New(0, 4, 1, 5)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4)},
+				cpuset.New(0, 4)},
 		},
 		{
 			description:      "Multi Init Container More CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID-1", "initFakeID-2"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
@@ -431,17 +512,17 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 				[]struct{ request, limit string }{
 					{"4000m", "4000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4),
-				cpuset.NewCPUSet(0, 4)},
+				cpuset.New(0, 4),
+				cpuset.New(0, 4)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4, 1, 5)},
 		},
 		{
 			description:      "Multi Init Container Increasing CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID-1", "initFakeID-2"},
 			containerIDs:     []string{"appFakeID"},
 			pod: makeMultiContainerPod(
@@ -451,17 +532,17 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 				[]struct{ request, limit string }{
 					{"6000m", "6000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4),
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4),
+				cpuset.New(0, 4, 1, 5)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4, 1, 5, 2, 6)},
+				cpuset.New(0, 4, 1, 5, 2, 6)},
 		},
 		{
 			description:      "Multi Init, Multi App Container Split CPUs",
 			topo:             topoSingleSocketHT,
 			numReservedCPUs:  0,
 			stAssignments:    state.ContainerCPUAssignments{},
-			stDefaultCPUSet:  cpuset.NewCPUSet(0, 1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:  cpuset.New(0, 1, 2, 3, 4, 5, 6, 7),
 			initContainerIDs: []string{"initFakeID-1", "initFakeID-2"},
 			containerIDs:     []string{"appFakeID-1", "appFakeID-2"},
 			pod: makeMultiContainerPod(
@@ -472,16 +553,17 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 					{"2000m", "2000m"},
 					{"2000m", "2000m"}}),
 			expInitCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4),
-				cpuset.NewCPUSet(0, 4, 1, 5)},
+				cpuset.New(0, 4),
+				cpuset.New(0, 4, 1, 5)},
 			expCSets: []cpuset.CPUSet{
-				cpuset.NewCPUSet(0, 4),
-				cpuset.NewCPUSet(1, 5)},
+				cpuset.New(0, 4),
+				cpuset.New(1, 5)},
 		},
 	}
 
 	for _, testCase := range testCases {
-		policy, _ := NewStaticPolicy(testCase.topo, testCase.numReservedCPUs, cpuset.NewCPUSet(), topologymanager.NewFakeManager(), nil)
+		logger, _ := ktesting.NewTestContext(t)
+		policy, _ := NewStaticPolicy(logger, testCase.topo, testCase.numReservedCPUs, cpuset.New(), topologymanager.NewFakeManager(), nil)
 
 		mockState := &mockState{
 			assignments:   testCase.stAssignments,
@@ -491,7 +573,7 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 		mgr := &manager{
 			policy:            policy,
 			state:             mockState,
-			lastUpdateState:   state.NewMemoryState(),
+			lastUpdateState:   state.NewMemoryState(logger),
 			containerRuntime:  mockRuntimeService{},
 			containerMap:      containermap.NewContainerMap(),
 			podStatusProvider: mockPodStatusProvider{},
@@ -513,7 +595,7 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 			testCase.expInitCSets,
 			testCase.expCSets...)
 
-		cumCSet := cpuset.NewCPUSet()
+		cumCSet := cpuset.New()
 
 		for i := range containers {
 			err := mgr.Allocate(testCase.pod, &containers[i])
@@ -522,7 +604,7 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 					testCase.description, containerIDs[i], err)
 			}
 
-			mgr.AddContainer(testCase.pod, &containers[i], containerIDs[i])
+			mgr.AddContainer(logger, testCase.pod, &containers[i], containerIDs[i])
 			_, _, err = mgr.containerMap.GetContainerRef(containerIDs[i])
 			if err != nil {
 				t.Errorf("StaticPolicy AddContainer() error (%v). unexpected error for container id: %v: %v",
@@ -551,6 +633,10 @@ func TestCPUManagerAddWithInitContainers(t *testing.T) {
 }
 
 func TestCPUManagerGenerate(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
 	testCases := []struct {
 		description                string
 		cpuPolicyName              string
@@ -604,20 +690,24 @@ func TestCPUManagerGenerate(t *testing.T) {
 			{
 				Cores: []cadvisorapi.Core{
 					{
-						Id:      0,
-						Threads: []int{0},
+						Id:           0,
+						Threads:      []int{0},
+						UncoreCaches: []cadvisorapi.Cache{{Id: 1}},
 					},
 					{
-						Id:      1,
-						Threads: []int{1},
+						Id:           1,
+						Threads:      []int{1},
+						UncoreCaches: []cadvisorapi.Cache{{Id: 1}},
 					},
 					{
-						Id:      2,
-						Threads: []int{2},
+						Id:           2,
+						Threads:      []int{2},
+						UncoreCaches: []cadvisorapi.Cache{{Id: 1}},
 					},
 					{
-						Id:      3,
-						Threads: []int{3},
+						Id:           3,
+						Threads:      []int{3},
+						UncoreCaches: []cadvisorapi.Cache{{Id: 1}},
 					},
 				},
 			},
@@ -630,13 +720,14 @@ func TestCPUManagerGenerate(t *testing.T) {
 			if testCase.isTopologyBroken {
 				machineInfo = &cadvisorapi.MachineInfo{}
 			}
-			sDir, err := ioutil.TempDir("/tmp/", "cpu_manager_test")
+			sDir, err := os.MkdirTemp("", "cpu_manager_test")
 			if err != nil {
 				t.Errorf("cannot create state file: %s", err.Error())
 			}
 			defer os.RemoveAll(sDir)
 
-			mgr, err := NewManager(testCase.cpuPolicyName, nil, 5*time.Second, machineInfo, cpuset.NewCPUSet(), testCase.nodeAllocatableReservation, sDir, topologymanager.NewFakeManager())
+			logger, _ := ktesting.NewTestContext(t)
+			mgr, err := NewManager(logger, testCase.cpuPolicyName, nil, 5*time.Second, machineInfo, cpuset.New(), testCase.nodeAllocatableReservation, sDir, topologymanager.NewFakeManager())
 			if testCase.expectedError != nil {
 				if !strings.Contains(err.Error(), testCase.expectedError.Error()) {
 					t.Errorf("Unexpected error message. Have: %s wants %s", err.Error(), testCase.expectedError.Error())
@@ -646,15 +737,8 @@ func TestCPUManagerGenerate(t *testing.T) {
 				if rawMgr.policy.Name() != testCase.expectedPolicy {
 					t.Errorf("Unexpected policy name. Have: %q wants %q", rawMgr.policy.Name(), testCase.expectedPolicy)
 				}
-				if rawMgr.policy.Name() == string(PolicyNone) {
-					if rawMgr.topology != nil {
-						t.Errorf("Expected topology to be nil for 'none' policy. Have: %q", rawMgr.topology)
-					}
-				}
-				if rawMgr.policy.Name() != string(PolicyNone) {
-					if rawMgr.topology == nil {
-						t.Errorf("Expected topology to be non-nil for policy '%v'. Have: %q", rawMgr.policy.Name(), rawMgr.topology)
-					}
+				if rawMgr.topology == nil {
+					t.Errorf("Expected topology to be non-nil for policy '%v'. Have: %q", rawMgr.policy.Name(), rawMgr.topology)
 				}
 			}
 		})
@@ -663,18 +747,23 @@ func TestCPUManagerGenerate(t *testing.T) {
 }
 
 func TestCPUManagerRemove(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
 	containerID := "fakeID"
 	containerMap := containermap.NewContainerMap()
 
+	logger, _ := ktesting.NewTestContext(t)
 	mgr := &manager{
 		policy: &mockPolicy{
 			err: nil,
 		},
 		state: &mockState{
 			assignments:   state.ContainerCPUAssignments{},
-			defaultCPUSet: cpuset.NewCPUSet(),
+			defaultCPUSet: cpuset.New(),
 		},
-		lastUpdateState:   state.NewMemoryState(),
+		lastUpdateState:   state.NewMemoryState(logger),
 		containerRuntime:  mockRuntimeService{},
 		containerMap:      containerMap,
 		activePods:        func() []*v1.Pod { return nil },
@@ -682,7 +771,7 @@ func TestCPUManagerRemove(t *testing.T) {
 	}
 
 	containerMap.Add("", "", containerID)
-	err := mgr.RemoveContainer(containerID)
+	err := mgr.RemoveContainer(logger, containerID)
 	if err != nil {
 		t.Errorf("CPU Manager RemoveContainer() error. expected error to be nil but got: %v", err)
 	}
@@ -691,7 +780,7 @@ func TestCPUManagerRemove(t *testing.T) {
 		policy: &mockPolicy{
 			err: fmt.Errorf("fake error"),
 		},
-		state:             state.NewMemoryState(),
+		state:             state.NewMemoryState(logger),
 		containerRuntime:  mockRuntimeService{},
 		containerMap:      containerMap,
 		activePods:        func() []*v1.Pod { return nil },
@@ -699,14 +788,21 @@ func TestCPUManagerRemove(t *testing.T) {
 	}
 
 	containerMap.Add("", "", containerID)
-	err = mgr.RemoveContainer(containerID)
+	err = mgr.RemoveContainer(logger, containerID)
 	if !reflect.DeepEqual(err, fmt.Errorf("fake error")) {
 		t.Errorf("CPU Manager RemoveContainer() error. expected error: fake error but got: %v", err)
 	}
 }
 
 func TestReconcileState(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
+	logger, _ := ktesting.NewTestContext(t)
+
 	testPolicy, _ := NewStaticPolicy(
+		logger,
 		&topology.CPUTopology{
 			NumCPUs:    8,
 			NumSockets: 2,
@@ -723,7 +819,7 @@ func TestReconcileState(t *testing.T) {
 			},
 		},
 		0,
-		cpuset.NewCPUSet(),
+		cpuset.New(),
 		topologymanager.NewFakeManager(),
 		nil)
 
@@ -776,18 +872,18 @@ func TestReconcileState(t *testing.T) {
 			updateErr: nil,
 			stAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			stDefaultCPUSet:           cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			stDefaultCPUSet:           cpuset.New(3, 4, 5, 6, 7),
 			lastUpdateStAssignments:   state.ContainerCPUAssignments{},
-			lastUpdateStDefaultCPUSet: cpuset.NewCPUSet(),
+			lastUpdateStDefaultCPUSet: cpuset.New(),
 			expectStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			expectStDefaultCPUSet:        cpuset.New(3, 4, 5, 6, 7),
 			expectSucceededContainerName: "fakeContainerName",
 			expectFailedContainerName:    "",
 		},
@@ -824,18 +920,18 @@ func TestReconcileState(t *testing.T) {
 			updateErr: nil,
 			stAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			stDefaultCPUSet:           cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			stDefaultCPUSet:           cpuset.New(3, 4, 5, 6, 7),
 			lastUpdateStAssignments:   state.ContainerCPUAssignments{},
-			lastUpdateStDefaultCPUSet: cpuset.NewCPUSet(),
+			lastUpdateStDefaultCPUSet: cpuset.New(),
 			expectStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			expectStDefaultCPUSet:        cpuset.New(3, 4, 5, 6, 7),
 			expectSucceededContainerName: "fakeContainerName",
 			expectFailedContainerName:    "",
 		},
@@ -861,11 +957,11 @@ func TestReconcileState(t *testing.T) {
 			pspFound:                     false,
 			updateErr:                    nil,
 			stAssignments:                state.ContainerCPUAssignments{},
-			stDefaultCPUSet:              cpuset.NewCPUSet(),
+			stDefaultCPUSet:              cpuset.New(),
 			lastUpdateStAssignments:      state.ContainerCPUAssignments{},
-			lastUpdateStDefaultCPUSet:    cpuset.NewCPUSet(),
+			lastUpdateStDefaultCPUSet:    cpuset.New(),
 			expectStAssignments:          state.ContainerCPUAssignments{},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(),
+			expectStDefaultCPUSet:        cpuset.New(),
 			expectSucceededContainerName: "",
 			expectFailedContainerName:    "",
 		},
@@ -898,11 +994,11 @@ func TestReconcileState(t *testing.T) {
 			pspFound:                     true,
 			updateErr:                    nil,
 			stAssignments:                state.ContainerCPUAssignments{},
-			stDefaultCPUSet:              cpuset.NewCPUSet(),
+			stDefaultCPUSet:              cpuset.New(),
 			lastUpdateStAssignments:      state.ContainerCPUAssignments{},
-			lastUpdateStDefaultCPUSet:    cpuset.NewCPUSet(),
+			lastUpdateStDefaultCPUSet:    cpuset.New(),
 			expectStAssignments:          state.ContainerCPUAssignments{},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(),
+			expectStDefaultCPUSet:        cpuset.New(),
 			expectSucceededContainerName: "",
 			expectFailedContainerName:    "fakeContainerName",
 		},
@@ -939,18 +1035,18 @@ func TestReconcileState(t *testing.T) {
 			updateErr: nil,
 			stAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(),
+					"fakeContainerName": cpuset.New(),
 				},
 			},
-			stDefaultCPUSet:           cpuset.NewCPUSet(1, 2, 3, 4, 5, 6, 7),
+			stDefaultCPUSet:           cpuset.New(1, 2, 3, 4, 5, 6, 7),
 			lastUpdateStAssignments:   state.ContainerCPUAssignments{},
-			lastUpdateStDefaultCPUSet: cpuset.NewCPUSet(),
+			lastUpdateStDefaultCPUSet: cpuset.New(),
 			expectStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(),
+					"fakeContainerName": cpuset.New(),
 				},
 			},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(1, 2, 3, 4, 5, 6, 7),
+			expectStDefaultCPUSet:        cpuset.New(1, 2, 3, 4, 5, 6, 7),
 			expectSucceededContainerName: "",
 			expectFailedContainerName:    "fakeContainerName",
 		},
@@ -987,18 +1083,18 @@ func TestReconcileState(t *testing.T) {
 			updateErr: fmt.Errorf("fake container update error"),
 			stAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			stDefaultCPUSet:           cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			stDefaultCPUSet:           cpuset.New(3, 4, 5, 6, 7),
 			lastUpdateStAssignments:   state.ContainerCPUAssignments{},
-			lastUpdateStDefaultCPUSet: cpuset.NewCPUSet(),
+			lastUpdateStDefaultCPUSet: cpuset.New(),
 			expectStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			expectStDefaultCPUSet:        cpuset.New(3, 4, 5, 6, 7),
 			expectSucceededContainerName: "",
 			expectFailedContainerName:    "fakeContainerName",
 		},
@@ -1035,21 +1131,21 @@ func TestReconcileState(t *testing.T) {
 			updateErr: nil,
 			stAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 				"secondfakePodUID": map[string]cpuset.CPUSet{
-					"secondfakeContainerName": cpuset.NewCPUSet(3, 4),
+					"secondfakeContainerName": cpuset.New(3, 4),
 				},
 			},
-			stDefaultCPUSet:           cpuset.NewCPUSet(5, 6, 7),
+			stDefaultCPUSet:           cpuset.New(5, 6, 7),
 			lastUpdateStAssignments:   state.ContainerCPUAssignments{},
-			lastUpdateStDefaultCPUSet: cpuset.NewCPUSet(),
+			lastUpdateStDefaultCPUSet: cpuset.New(),
 			expectStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			expectStDefaultCPUSet:        cpuset.New(3, 4, 5, 6, 7),
 			expectSucceededContainerName: "fakeContainerName",
 			expectFailedContainerName:    "",
 		},
@@ -1086,22 +1182,22 @@ func TestReconcileState(t *testing.T) {
 			updateErr: nil,
 			stAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			stDefaultCPUSet: cpuset.NewCPUSet(5, 6, 7),
+			stDefaultCPUSet: cpuset.New(5, 6, 7),
 			lastUpdateStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			lastUpdateStDefaultCPUSet: cpuset.NewCPUSet(5, 6, 7),
+			lastUpdateStDefaultCPUSet: cpuset.New(5, 6, 7),
 			expectStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(5, 6, 7),
+			expectStDefaultCPUSet:        cpuset.New(5, 6, 7),
 			expectSucceededContainerName: "fakeContainerName",
 			expectFailedContainerName:    "",
 		},
@@ -1138,35 +1234,36 @@ func TestReconcileState(t *testing.T) {
 			updateErr: nil,
 			stAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			stDefaultCPUSet: cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			stDefaultCPUSet: cpuset.New(3, 4, 5, 6, 7),
 			lastUpdateStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(3, 4),
+					"fakeContainerName": cpuset.New(3, 4),
 				},
 			},
-			lastUpdateStDefaultCPUSet: cpuset.NewCPUSet(1, 2, 5, 6, 7),
+			lastUpdateStDefaultCPUSet: cpuset.New(1, 2, 5, 6, 7),
 			expectStAssignments: state.ContainerCPUAssignments{
 				"fakePodUID": map[string]cpuset.CPUSet{
-					"fakeContainerName": cpuset.NewCPUSet(1, 2),
+					"fakeContainerName": cpuset.New(1, 2),
 				},
 			},
-			expectStDefaultCPUSet:        cpuset.NewCPUSet(3, 4, 5, 6, 7),
+			expectStDefaultCPUSet:        cpuset.New(3, 4, 5, 6, 7),
 			expectSucceededContainerName: "fakeContainerName",
 			expectFailedContainerName:    "",
 		},
 	}
 
 	for _, testCase := range testCases {
+		logger, _ := ktesting.NewTestContext(t)
 		mgr := &manager{
 			policy: testCase.policy,
 			state: &mockState{
 				assignments:   testCase.stAssignments,
 				defaultCPUSet: testCase.stDefaultCPUSet,
 			},
-			lastUpdateState: state.NewMemoryState(),
+			lastUpdateState: state.NewMemoryState(logger),
 			containerRuntime: mockRuntimeService{
 				err: testCase.updateErr,
 			},
@@ -1180,7 +1277,7 @@ func TestReconcileState(t *testing.T) {
 			},
 		}
 		mgr.sourcesReady = &sourcesReadyStub{}
-		success, failure := mgr.reconcileState()
+		success, failure := mgr.reconcileState(context.Background())
 
 		if !reflect.DeepEqual(testCase.expectStAssignments, mgr.state.GetCPUAssignments()) {
 			t.Errorf("%v", testCase.description)
@@ -1229,7 +1326,13 @@ func TestReconcileState(t *testing.T) {
 // above test cases are without kubelet --reserved-cpus cmd option
 // the following tests are with --reserved-cpus configured
 func TestCPUManagerAddWithResvList(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
+	logger, _ := ktesting.NewTestContext(t)
 	testPolicy, _ := NewStaticPolicy(
+		logger,
 		&topology.CPUTopology{
 			NumCPUs:    4,
 			NumSockets: 1,
@@ -1242,7 +1345,7 @@ func TestCPUManagerAddWithResvList(t *testing.T) {
 			},
 		},
 		1,
-		cpuset.NewCPUSet(0),
+		cpuset.New(0),
 		topologymanager.NewFakeManager(),
 		nil)
 	testCases := []struct {
@@ -1257,7 +1360,7 @@ func TestCPUManagerAddWithResvList(t *testing.T) {
 			description:        "cpu manager add - no error",
 			updateErr:          nil,
 			policy:             testPolicy,
-			expCPUSet:          cpuset.NewCPUSet(0, 3),
+			expCPUSet:          cpuset.New(0, 3),
 			expAllocateErr:     nil,
 			expAddContainerErr: nil,
 		},
@@ -1268,9 +1371,9 @@ func TestCPUManagerAddWithResvList(t *testing.T) {
 			policy: testCase.policy,
 			state: &mockState{
 				assignments:   state.ContainerCPUAssignments{},
-				defaultCPUSet: cpuset.NewCPUSet(0, 1, 2, 3),
+				defaultCPUSet: cpuset.New(0, 1, 2, 3),
 			},
-			lastUpdateState: state.NewMemoryState(),
+			lastUpdateState: state.NewMemoryState(logger),
 			containerRuntime: mockRuntimeService{
 				err: testCase.updateErr,
 			},
@@ -1281,7 +1384,7 @@ func TestCPUManagerAddWithResvList(t *testing.T) {
 
 		pod := makePod("fakePod", "fakeContainer", "2", "2")
 		container := &pod.Spec.Containers[0]
-		mgr.activePods = func() []*v1.Pod { return nil }
+		mgr.activePods = func() []*v1.Pod { return []*v1.Pod{pod} }
 
 		err := mgr.Allocate(pod, container)
 		if !reflect.DeepEqual(err, testCase.expAllocateErr) {
@@ -1289,7 +1392,7 @@ func TestCPUManagerAddWithResvList(t *testing.T) {
 				testCase.description, testCase.expAllocateErr, err)
 		}
 
-		mgr.AddContainer(pod, container, "fakeID")
+		mgr.AddContainer(logger, pod, container, "fakeID")
 		_, _, err = mgr.containerMap.GetContainerRef("fakeID")
 		if !reflect.DeepEqual(err, testCase.expAddContainerErr) {
 			t.Errorf("CPU Manager AddContainer() error (%v). expected error: %v but got: %v",
@@ -1303,6 +1406,10 @@ func TestCPUManagerAddWithResvList(t *testing.T) {
 }
 
 func TestCPUManagerHandlePolicyOptions(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
 	testCases := []struct {
 		description      string
 		cpuPolicyName    string
@@ -1350,13 +1457,14 @@ func TestCPUManagerHandlePolicyOptions(t *testing.T) {
 		t.Run(testCase.description, func(t *testing.T) {
 			machineInfo := &mockedMachineInfo
 			nodeAllocatableReservation := v1.ResourceList{}
-			sDir, err := ioutil.TempDir("/tmp/", "cpu_manager_test")
+			sDir, err := os.MkdirTemp("", "cpu_manager_test")
 			if err != nil {
 				t.Errorf("cannot create state file: %s", err.Error())
 			}
 			defer os.RemoveAll(sDir)
 
-			_, err = NewManager(testCase.cpuPolicyName, testCase.cpuPolicyOptions, 5*time.Second, machineInfo, cpuset.NewCPUSet(), nodeAllocatableReservation, sDir, topologymanager.NewFakeManager())
+			logger, _ := ktesting.NewTestContext(t)
+			_, err = NewManager(logger, testCase.cpuPolicyName, testCase.cpuPolicyOptions, 5*time.Second, machineInfo, cpuset.New(), nodeAllocatableReservation, sDir, topologymanager.NewFakeManager())
 			if err == nil {
 				t.Errorf("Expected error, but NewManager succeeded")
 			}
@@ -1369,8 +1477,14 @@ func TestCPUManagerHandlePolicyOptions(t *testing.T) {
 }
 
 func TestCPUManagerGetAllocatableCPUs(t *testing.T) {
+	logger, _ := ktesting.NewTestContext(t)
+	if runtime.GOOS == "windows" {
+		featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.WindowsCPUAndMemoryAffinity, true)
+	}
+
 	nonePolicy, _ := NewNonePolicy(nil)
 	staticPolicy, _ := NewStaticPolicy(
+		logger,
 		&topology.CPUTopology{
 			NumCPUs:    4,
 			NumSockets: 1,
@@ -1383,7 +1497,7 @@ func TestCPUManagerGetAllocatableCPUs(t *testing.T) {
 			},
 		},
 		1,
-		cpuset.NewCPUSet(0),
+		cpuset.New(0),
 		topologymanager.NewFakeManager(),
 		nil)
 
@@ -1395,12 +1509,12 @@ func TestCPUManagerGetAllocatableCPUs(t *testing.T) {
 		{
 			description:        "None Policy",
 			policy:             nonePolicy,
-			expAllocatableCPUs: cpuset.NewCPUSet(),
+			expAllocatableCPUs: cpuset.New(),
 		},
 		{
 			description:        "Static Policy",
 			policy:             staticPolicy,
-			expAllocatableCPUs: cpuset.NewCPUSet(1, 2, 3),
+			expAllocatableCPUs: cpuset.New(1, 2, 3),
 		},
 	}
 	for _, testCase := range testCases {
@@ -1409,9 +1523,9 @@ func TestCPUManagerGetAllocatableCPUs(t *testing.T) {
 			activePods: func() []*v1.Pod { return nil },
 			state: &mockState{
 				assignments:   state.ContainerCPUAssignments{},
-				defaultCPUSet: cpuset.NewCPUSet(0, 1, 2, 3),
+				defaultCPUSet: cpuset.New(0, 1, 2, 3),
 			},
-			lastUpdateState:   state.NewMemoryState(),
+			lastUpdateState:   state.NewMemoryState(logger),
 			containerMap:      containermap.NewContainerMap(),
 			podStatusProvider: mockPodStatusProvider{},
 			sourcesReady:      &sourcesReadyStub{},
