@@ -19,6 +19,7 @@ package eviction
 import (
 	"context"
 	"fmt"
+	"runtime"
 	"sort"
 	"sync"
 	"time"
@@ -33,8 +34,8 @@ import (
 	statsapi "k8s.io/kubelet/pkg/apis/stats/v1alpha1"
 	"k8s.io/utils/clock"
 
+	resourcehelper "k8s.io/component-helpers/resource"
 	podutil "k8s.io/kubernetes/pkg/api/v1/pod"
-	resourcehelper "k8s.io/kubernetes/pkg/api/v1/resource"
 	v1qos "k8s.io/kubernetes/pkg/apis/core/v1/helper/qos"
 	"k8s.io/kubernetes/pkg/features"
 	evictionapi "k8s.io/kubernetes/pkg/kubelet/eviction/api"
@@ -56,6 +57,9 @@ const (
 	signalEphemeralPodFsLimit string = "ephemeralpodfs.limit"
 	// signalEmptyDirFsLimit is amount of storage available on filesystem requested by an emptyDir
 	signalEmptyDirFsLimit string = "emptydirfs.limit"
+	// immediateEvictionGracePeriodSeconds is how long we give pods to shut down when we
+	// need them to evict quickly due to resource pressure
+	immediateEvictionGracePeriodSeconds = 1
 )
 
 // managerImpl implements Manager
@@ -142,6 +146,10 @@ func NewManager(
 func (m *managerImpl) Admit(attrs *lifecycle.PodAdmitAttributes) lifecycle.PodAdmitResult {
 	m.RLock()
 	defer m.RUnlock()
+
+	ctx := context.Background()
+	logger := klog.FromContext(ctx)
+
 	if len(m.nodeConditions) == 0 {
 		return lifecycle.PodAdmitResult{Admit: true}
 	}
@@ -161,16 +169,14 @@ func (m *managerImpl) Admit(attrs *lifecycle.PodAdmitAttributes) lifecycle.PodAd
 
 		// When node has memory pressure, check BestEffort Pod's toleration:
 		// admit it if tolerates memory pressure taint, fail for other tolerations, e.g. DiskPressure.
-		if corev1helpers.TolerationsTolerateTaint(attrs.Pod.Spec.Tolerations, &v1.Taint{
+		if corev1helpers.TolerationsTolerateTaint(logger, attrs.Pod.Spec.Tolerations, &v1.Taint{
 			Key:    v1.TaintNodeMemoryPressure,
 			Effect: v1.TaintEffectNoSchedule,
-		}) {
+		}, utilfeature.DefaultFeatureGate.Enabled(features.TaintTolerationComparisonOperators)) {
 			return lifecycle.PodAdmitResult{Admit: true}
 		}
 	}
 
-	// reject pods when under memory pressure (if pod is best effort), or if under disk pressure.
-	klog.InfoS("Failed to admit pod to node", "pod", klog.KObj(attrs.Pod), "nodeCondition", m.nodeConditions)
 	return lifecycle.PodAdmitResult{
 		Admit:   false,
 		Reason:  Reason,
@@ -179,19 +185,21 @@ func (m *managerImpl) Admit(attrs *lifecycle.PodAdmitAttributes) lifecycle.PodAd
 }
 
 // Start starts the control loop to observe and response to low compute resources.
-func (m *managerImpl) Start(diskInfoProvider DiskInfoProvider, podFunc ActivePodsFunc, podCleanedUpFunc PodCleanedUpFunc, monitoringInterval time.Duration) {
+func (m *managerImpl) Start(ctx context.Context, diskInfoProvider DiskInfoProvider, podFunc ActivePodsFunc, podCleanedUpFunc PodCleanedUpFunc, monitoringInterval time.Duration) {
+	logger := klog.FromContext(ctx)
 	thresholdHandler := func(message string) {
-		klog.InfoS(message)
-		m.synchronize(diskInfoProvider, podFunc)
+		logger.Info(message)
+		_, _ = m.synchronize(ctx, diskInfoProvider, podFunc)
 	}
-	if m.config.KernelMemcgNotification {
+	logger.Info("Eviction manager: starting control loop")
+	if m.config.KernelMemcgNotification || runtime.GOOS == "windows" {
 		for _, threshold := range m.config.Thresholds {
 			if threshold.Signal == evictionapi.SignalMemoryAvailable || threshold.Signal == evictionapi.SignalAllocatableMemoryAvailable {
-				notifier, err := NewMemoryThresholdNotifier(threshold, m.config.PodCgroupRoot, &CgroupNotifierFactory{}, thresholdHandler)
+				notifier, err := NewMemoryThresholdNotifier(logger, threshold, m.config.PodCgroupRoot, &CgroupNotifierFactory{}, thresholdHandler)
 				if err != nil {
-					klog.InfoS("Eviction manager: failed to create memory threshold notifier", "err", err)
+					logger.Info("Eviction manager: failed to create memory threshold notifier", "err", err)
 				} else {
-					go notifier.Start()
+					go notifier.Start(ctx)
 					m.thresholdNotifiers = append(m.thresholdNotifiers, notifier)
 				}
 			}
@@ -200,13 +208,13 @@ func (m *managerImpl) Start(diskInfoProvider DiskInfoProvider, podFunc ActivePod
 	// start the eviction manager monitoring
 	go func() {
 		for {
-			evictedPods, err := m.synchronize(diskInfoProvider, podFunc)
+			evictedPods, err := m.synchronize(ctx, diskInfoProvider, podFunc)
 			if evictedPods != nil && err == nil {
-				klog.InfoS("Eviction manager: pods evicted, waiting for pod to be cleaned up", "pods", klog.KObjSlice(evictedPods))
-				m.waitForPodsCleanup(podCleanedUpFunc, evictedPods)
+				logger.Info("Eviction manager: pods evicted, waiting for pod to be cleaned up", "pods", klog.KObjSlice(evictedPods))
+				m.waitForPodsCleanup(logger, podCleanedUpFunc, evictedPods)
 			} else {
 				if err != nil {
-					klog.ErrorS(err, "Eviction manager: failed to synchronize")
+					logger.Error(err, "Eviction manager: failed to synchronize")
 				}
 				time.Sleep(monitoringInterval)
 			}
@@ -237,25 +245,32 @@ func (m *managerImpl) IsUnderPIDPressure() bool {
 
 // synchronize is the main control loop that enforces eviction thresholds.
 // Returns the pod that was killed, or nil if no pod was killed.
-func (m *managerImpl) synchronize(diskInfoProvider DiskInfoProvider, podFunc ActivePodsFunc) ([]*v1.Pod, error) {
-	ctx := context.Background()
+func (m *managerImpl) synchronize(ctx context.Context, diskInfoProvider DiskInfoProvider, podFunc ActivePodsFunc) ([]*v1.Pod, error) {
+	logger := klog.FromContext(ctx)
 	// if we have nothing to do, just return
 	thresholds := m.config.Thresholds
 	if len(thresholds) == 0 && !m.localStorageCapacityIsolation {
 		return nil, nil
 	}
 
-	klog.V(3).InfoS("Eviction manager: synchronize housekeeping")
+	logger.V(3).Info("Eviction manager: synchronize housekeeping")
 	// build the ranking functions (if not yet known)
 	// TODO: have a function in cadvisor that lets us know if global housekeeping has completed
 	if m.dedicatedImageFs == nil {
-		hasImageFs, splitDiskError := diskInfoProvider.HasDedicatedImageFs(ctx)
-		if splitDiskError != nil {
-			klog.ErrorS(splitDiskError, "Eviction manager: failed to get HasDedicatedImageFs")
-			return nil, fmt.Errorf("eviction manager: failed to get HasDedicatedImageFs: %v", splitDiskError)
+		hasImageFs, imageFsErr := diskInfoProvider.HasDedicatedImageFs(ctx)
+		if imageFsErr != nil {
+			// TODO: This should be refactored to log an error and retry the HasDedicatedImageFs
+			// If we have a transient error this will never be retried and we will not set eviction signals
+			logger.Error(imageFsErr, "Eviction manager: failed to get HasDedicatedImageFs")
+			return nil, fmt.Errorf("eviction manager: failed to get HasDedicatedImageFs: %w", imageFsErr)
 		}
 		m.dedicatedImageFs = &hasImageFs
-		splitContainerImageFs := m.containerGC.IsContainerFsSeparateFromImageFs(ctx)
+		splitContainerImageFs, splitErr := diskInfoProvider.HasDedicatedContainerFs(ctx)
+		if splitErr != nil {
+			// A common error case is when there is no split filesystem
+			// there is an error finding the split filesystem label and we want to ignore these errors
+			logger.Error(splitErr, "eviction manager: failed to check if we have separate container filesystem. Ignoring.")
+		}
 
 		// If we are a split filesystem but the feature is turned off
 		// we should return an error.
@@ -267,45 +282,45 @@ func (m *managerImpl) synchronize(diskInfoProvider DiskInfoProvider, podFunc Act
 		thresholds, err := UpdateContainerFsThresholds(m.config.Thresholds, hasImageFs, splitContainerImageFs)
 		m.config.Thresholds = thresholds
 		if err != nil {
-			klog.ErrorS(err, "eviction manager: found conflicting containerfs eviction. Ignoring.")
+			logger.Error(err, "eviction manager: found conflicting containerfs eviction. Ignoring.")
 		}
 		m.splitContainerImageFs = &splitContainerImageFs
 		m.signalToRankFunc = buildSignalToRankFunc(hasImageFs, splitContainerImageFs)
 		m.signalToNodeReclaimFuncs = buildSignalToNodeReclaimFuncs(m.imageGC, m.containerGC, hasImageFs, splitContainerImageFs)
 	}
 
-	klog.V(3).InfoS("FileSystem detection", "DedicatedImageFs", m.dedicatedImageFs, "SplitImageFs", m.splitContainerImageFs)
+	logger.V(3).Info("FileSystem detection", "DedicatedImageFs", m.dedicatedImageFs, "SplitImageFs", m.splitContainerImageFs)
 	activePods := podFunc()
 	updateStats := true
 	summary, err := m.summaryProvider.Get(ctx, updateStats)
 	if err != nil {
-		klog.ErrorS(err, "Eviction manager: failed to get summary stats")
+		logger.Error(err, "Eviction manager: failed to get summary stats")
 		return nil, nil
 	}
 
 	if m.clock.Since(m.thresholdsLastUpdated) > notifierRefreshInterval {
 		m.thresholdsLastUpdated = m.clock.Now()
 		for _, notifier := range m.thresholdNotifiers {
-			if err := notifier.UpdateThreshold(summary); err != nil {
-				klog.InfoS("Eviction manager: failed to update notifier", "notifier", notifier.Description(), "err", err)
+			if err := notifier.UpdateThreshold(ctx, summary); err != nil {
+				logger.Info("Eviction manager: failed to update notifier", "notifier", notifier.Description(), "err", err)
 			}
 		}
 	}
 
 	// make observations and get a function to derive pod usage stats relative to those observations.
-	observations, statsFunc := makeSignalObservations(summary)
-	debugLogObservations("observations", observations)
+	observations, statsFunc := makeSignalObservations(logger, summary)
+	debugLogObservations(logger, "observations", observations)
 
 	// determine the set of thresholds met independent of grace period
-	thresholds = thresholdsMet(thresholds, observations, false)
-	debugLogThresholdsWithObservation("thresholds - ignoring grace period", thresholds, observations)
+	thresholds = thresholdsMet(logger, thresholds, observations, false)
+	debugLogThresholdsWithObservation(logger, "thresholds - ignoring grace period", thresholds, observations)
 
 	// determine the set of thresholds previously met that have not yet satisfied the associated min-reclaim
 	if len(m.thresholdsMet) > 0 {
-		thresholdsNotYetResolved := thresholdsMet(m.thresholdsMet, observations, true)
+		thresholdsNotYetResolved := thresholdsMet(logger, m.thresholdsMet, observations, true)
 		thresholds = mergeThresholds(thresholds, thresholdsNotYetResolved)
 	}
-	debugLogThresholdsWithObservation("thresholds - reclaim not satisfied", thresholds, observations)
+	debugLogThresholdsWithObservation(logger, "thresholds - reclaim not satisfied", thresholds, observations)
 
 	// track when a threshold was first observed
 	now := m.clock.Now()
@@ -314,7 +329,7 @@ func (m *managerImpl) synchronize(diskInfoProvider DiskInfoProvider, podFunc Act
 	// the set of node conditions that are triggered by currently observed thresholds
 	nodeConditions := nodeConditions(thresholds)
 	if len(nodeConditions) > 0 {
-		klog.V(3).InfoS("Eviction manager: node conditions - observed", "nodeCondition", nodeConditions)
+		logger.V(3).Info("Eviction manager: node conditions - observed", "nodeCondition", nodeConditions)
 	}
 
 	// track when a node condition was last observed
@@ -323,12 +338,12 @@ func (m *managerImpl) synchronize(diskInfoProvider DiskInfoProvider, podFunc Act
 	// node conditions report true if it has been observed within the transition period window
 	nodeConditions = nodeConditionsObservedSince(nodeConditionsLastObservedAt, m.config.PressureTransitionPeriod, now)
 	if len(nodeConditions) > 0 {
-		klog.V(3).InfoS("Eviction manager: node conditions - transition period not met", "nodeCondition", nodeConditions)
+		logger.V(3).Info("Eviction manager: node conditions - transition period not met", "nodeCondition", nodeConditions)
 	}
 
 	// determine the set of thresholds we need to drive eviction behavior (i.e. all grace periods are met)
-	thresholds = thresholdsMetGracePeriod(thresholdsFirstObservedAt, now)
-	debugLogThresholdsWithObservation("thresholds - grace periods satisfied", thresholds, observations)
+	thresholds = thresholdsMetGracePeriod(logger, thresholdsFirstObservedAt, now)
+	debugLogThresholdsWithObservation(logger, "thresholds - grace periods satisfied", thresholds, observations)
 
 	// update internal state
 	m.Lock()
@@ -338,8 +353,8 @@ func (m *managerImpl) synchronize(diskInfoProvider DiskInfoProvider, podFunc Act
 	m.thresholdsMet = thresholds
 
 	// determine the set of thresholds whose stats have been updated since the last sync
-	thresholds = thresholdsUpdatedStats(thresholds, observations, m.lastObservations)
-	debugLogThresholdsWithObservation("thresholds - updated stats", thresholds, observations)
+	thresholds = thresholdsUpdatedStats(logger, thresholds, observations, m.lastObservations)
+	debugLogThresholdsWithObservation(logger, "thresholds - updated stats", thresholds, observations)
 
 	m.lastObservations = observations
 	m.Unlock()
@@ -347,52 +362,52 @@ func (m *managerImpl) synchronize(diskInfoProvider DiskInfoProvider, podFunc Act
 	// evict pods if there is a resource usage violation from local volume temporary storage
 	// If eviction happens in localStorageEviction function, skip the rest of eviction action
 	if m.localStorageCapacityIsolation {
-		if evictedPods := m.localStorageEviction(activePods, statsFunc); len(evictedPods) > 0 {
+		if evictedPods := m.localStorageEviction(logger, activePods, statsFunc); len(evictedPods) > 0 {
 			return evictedPods, nil
 		}
 	}
 
 	if len(thresholds) == 0 {
-		klog.V(3).InfoS("Eviction manager: no resources are starved")
+		logger.V(3).Info("Eviction manager: no resources are starved")
 		return nil, nil
 	}
 
 	// rank the thresholds by eviction priority
 	sort.Sort(byEvictionPriority(thresholds))
-	thresholdToReclaim, resourceToReclaim, foundAny := getReclaimableThreshold(thresholds)
+	thresholdToReclaim, resourceToReclaim, foundAny := getReclaimableThreshold(logger, thresholds)
 	if !foundAny {
 		return nil, nil
 	}
-	klog.InfoS("Eviction manager: attempting to reclaim", "resourceName", resourceToReclaim)
+	logger.Info("Eviction manager: attempting to reclaim", "resourceName", resourceToReclaim)
 
 	// record an event about the resources we are now attempting to reclaim via eviction
 	m.recorder.Eventf(m.nodeRef, v1.EventTypeWarning, "EvictionThresholdMet", "Attempting to reclaim %s", resourceToReclaim)
 
 	// check if there are node-level resources we can reclaim to reduce pressure before evicting end-user pods.
 	if m.reclaimNodeLevelResources(ctx, thresholdToReclaim.Signal, resourceToReclaim) {
-		klog.InfoS("Eviction manager: able to reduce resource pressure without evicting pods.", "resourceName", resourceToReclaim)
+		logger.Info("Eviction manager: able to reduce resource pressure without evicting pods.", "resourceName", resourceToReclaim)
 		return nil, nil
 	}
 
-	klog.InfoS("Eviction manager: must evict pod(s) to reclaim", "resourceName", resourceToReclaim)
+	logger.Info("Eviction manager: must evict pod(s) to reclaim", "resourceName", resourceToReclaim)
 
 	// rank the pods for eviction
 	rank, ok := m.signalToRankFunc[thresholdToReclaim.Signal]
 	if !ok {
-		klog.ErrorS(nil, "Eviction manager: no ranking function for signal", "threshold", thresholdToReclaim.Signal)
+		logger.Error(nil, "Eviction manager: no ranking function for signal", "threshold", thresholdToReclaim.Signal)
 		return nil, nil
 	}
 
 	// the only candidates viable for eviction are those pods that had anything running.
 	if len(activePods) == 0 {
-		klog.ErrorS(nil, "Eviction manager: eviction thresholds have been met, but no pods are active to evict")
+		logger.Error(nil, "Eviction manager: eviction thresholds have been met, but no pods are active to evict")
 		return nil, nil
 	}
 
 	// rank the running pods for eviction for the specified resource
 	rank(activePods, statsFunc)
 
-	klog.InfoS("Eviction manager: pods ranked for eviction", "pods", klog.KObjSlice(activePods))
+	logger.Info("Eviction manager: pods ranked for eviction", "pods", klog.KObjSlice(activePods))
 
 	//record age of metrics for met thresholds that we are using for evictions.
 	for _, t := range thresholds {
@@ -405,30 +420,32 @@ func (m *managerImpl) synchronize(diskInfoProvider DiskInfoProvider, podFunc Act
 	// we kill at most a single pod during each eviction interval
 	for i := range activePods {
 		pod := activePods[i]
-		gracePeriodOverride := int64(0)
+		gracePeriodOverride := int64(immediateEvictionGracePeriodSeconds)
 		if !isHardEvictionThreshold(thresholdToReclaim) {
 			gracePeriodOverride = m.config.MaxPodGracePeriodSeconds
-		}
-		message, annotations := evictionMessage(resourceToReclaim, pod, statsFunc, thresholds, observations)
-		var condition *v1.PodCondition
-		if utilfeature.DefaultFeatureGate.Enabled(features.PodDisruptionConditions) {
-			condition = &v1.PodCondition{
-				Type:    v1.DisruptionTarget,
-				Status:  v1.ConditionTrue,
-				Reason:  v1.PodReasonTerminationByKubelet,
-				Message: message,
+			if pod.Spec.TerminationGracePeriodSeconds != nil {
+				gracePeriodOverride = min(m.config.MaxPodGracePeriodSeconds, *pod.Spec.TerminationGracePeriodSeconds)
 			}
 		}
-		if m.evictPod(pod, gracePeriodOverride, message, annotations, condition) {
+
+		message, annotations := evictionMessage(resourceToReclaim, pod, statsFunc, thresholds, observations)
+		condition := &v1.PodCondition{
+			Type:               v1.DisruptionTarget,
+			ObservedGeneration: pod.Generation,
+			Status:             v1.ConditionTrue,
+			Reason:             v1.PodReasonTerminationByKubelet,
+			Message:            message,
+		}
+		if m.evictPod(logger, pod, gracePeriodOverride, message, annotations, condition) {
 			metrics.Evictions.WithLabelValues(string(thresholdToReclaim.Signal)).Inc()
 			return []*v1.Pod{pod}, nil
 		}
 	}
-	klog.InfoS("Eviction manager: unable to evict any pods from the node")
+	logger.Info("Eviction manager: unable to evict any pods from the node")
 	return nil, nil
 }
 
-func (m *managerImpl) waitForPodsCleanup(podCleanedUpFunc PodCleanedUpFunc, pods []*v1.Pod) {
+func (m *managerImpl) waitForPodsCleanup(logger klog.Logger, podCleanedUpFunc PodCleanedUpFunc, pods []*v1.Pod) {
 	timeout := m.clock.NewTimer(podCleanupTimeout)
 	defer timeout.Stop()
 	ticker := m.clock.NewTicker(podCleanupPollFreq)
@@ -436,7 +453,7 @@ func (m *managerImpl) waitForPodsCleanup(podCleanedUpFunc PodCleanedUpFunc, pods
 	for {
 		select {
 		case <-timeout.C():
-			klog.InfoS("Eviction manager: timed out waiting for pods to be cleaned up", "pods", klog.KObjSlice(pods))
+			logger.Info("Eviction manager: timed out waiting for pods to be cleaned up", "pods", klog.KObjSlice(pods))
 			return
 		case <-ticker.C():
 			for i, pod := range pods {
@@ -444,7 +461,7 @@ func (m *managerImpl) waitForPodsCleanup(podCleanedUpFunc PodCleanedUpFunc, pods
 					break
 				}
 				if i == len(pods)-1 {
-					klog.InfoS("Eviction manager: pods successfully cleaned up", "pods", klog.KObjSlice(pods))
+					logger.Info("Eviction manager: pods successfully cleaned up", "pods", klog.KObjSlice(pods))
 					return
 				}
 			}
@@ -454,29 +471,30 @@ func (m *managerImpl) waitForPodsCleanup(podCleanedUpFunc PodCleanedUpFunc, pods
 
 // reclaimNodeLevelResources attempts to reclaim node level resources.  returns true if thresholds were satisfied and no pod eviction is required.
 func (m *managerImpl) reclaimNodeLevelResources(ctx context.Context, signalToReclaim evictionapi.Signal, resourceToReclaim v1.ResourceName) bool {
+	logger := klog.FromContext(ctx)
 	nodeReclaimFuncs := m.signalToNodeReclaimFuncs[signalToReclaim]
 	for _, nodeReclaimFunc := range nodeReclaimFuncs {
 		// attempt to reclaim the pressured resource.
 		if err := nodeReclaimFunc(ctx); err != nil {
-			klog.InfoS("Eviction manager: unexpected error when attempting to reduce resource pressure", "resourceName", resourceToReclaim, "err", err)
+			logger.Info("Eviction manager: unexpected error when attempting to reduce resource pressure", "resourceName", resourceToReclaim, "err", err)
 		}
 
 	}
 	if len(nodeReclaimFuncs) > 0 {
 		summary, err := m.summaryProvider.Get(ctx, true)
 		if err != nil {
-			klog.ErrorS(err, "Eviction manager: failed to get summary stats after resource reclaim")
+			logger.Error(err, "Eviction manager: failed to get summary stats after resource reclaim")
 			return false
 		}
 
 		// make observations and get a function to derive pod usage stats relative to those observations.
-		observations, _ := makeSignalObservations(summary)
-		debugLogObservations("observations after resource reclaim", observations)
+		observations, _ := makeSignalObservations(logger, summary)
+		debugLogObservations(logger, "observations after resource reclaim", observations)
 
 		// evaluate all thresholds independently of their grace period to see if with
 		// the new observations, we think we have met min reclaim goals
-		thresholds := thresholdsMet(m.config.Thresholds, observations, true)
-		debugLogThresholdsWithObservation("thresholds after resource reclaim - ignoring grace period", thresholds, observations)
+		thresholds := thresholdsMet(logger, m.config.Thresholds, observations, true)
+		debugLogThresholdsWithObservation(logger, "thresholds after resource reclaim - ignoring grace period", thresholds, observations)
 
 		if len(thresholds) == 0 {
 			return true
@@ -487,7 +505,7 @@ func (m *managerImpl) reclaimNodeLevelResources(ctx context.Context, signalToRec
 
 // localStorageEviction checks the EmptyDir volume usage for each pod and determine whether it exceeds the specified limit and needs
 // to be evicted. It also checks every container in the pod, if the container overlay usage exceeds the limit, the pod will be evicted too.
-func (m *managerImpl) localStorageEviction(pods []*v1.Pod, statsFunc statsFunc) []*v1.Pod {
+func (m *managerImpl) localStorageEviction(logger klog.Logger, pods []*v1.Pod, statsFunc statsFunc) []*v1.Pod {
 	evicted := []*v1.Pod{}
 	for _, pod := range pods {
 		podStats, ok := statsFunc(pod)
@@ -495,17 +513,17 @@ func (m *managerImpl) localStorageEviction(pods []*v1.Pod, statsFunc statsFunc)
 			continue
 		}
 
-		if m.emptyDirLimitEviction(podStats, pod) {
+		if m.emptyDirLimitEviction(logger, podStats, pod) {
 			evicted = append(evicted, pod)
 			continue
 		}
 
-		if m.podEphemeralStorageLimitEviction(podStats, pod) {
+		if m.podEphemeralStorageLimitEviction(logger, podStats, pod) {
 			evicted = append(evicted, pod)
 			continue
 		}
 
-		if m.containerEphemeralStorageLimitEviction(podStats, pod) {
+		if m.containerEphemeralStorageLimitEviction(logger, podStats, pod) {
 			evicted = append(evicted, pod)
 		}
 	}
@@ -513,7 +531,7 @@ func (m *managerImpl) localStorageEviction(pods []*v1.Pod, statsFunc statsFunc)
 	return evicted
 }
 
-func (m *managerImpl) emptyDirLimitEviction(podStats statsapi.PodStats, pod *v1.Pod) bool {
+func (m *managerImpl) emptyDirLimitEviction(logger klog.Logger, podStats statsapi.PodStats, pod *v1.Pod) bool {
 	podVolumeUsed := make(map[string]*resource.Quantity)
 	for _, volume := range podStats.VolumeStats {
 		podVolumeUsed[volume.Name] = resource.NewQuantity(int64(*volume.UsedBytes), resource.BinarySI)
@@ -525,7 +543,7 @@ func (m *managerImpl) emptyDirLimitEviction(podStats statsapi.PodStats, pod *v1.
 			used := podVolumeUsed[pod.Spec.Volumes[i].Name]
 			if used != nil && size != nil && size.Sign() == 1 && used.Cmp(*size) > 0 {
 				// the emptyDir usage exceeds the size limit, evict the pod
-				if m.evictPod(pod, 0, fmt.Sprintf(emptyDirMessageFmt, pod.Spec.Volumes[i].Name, size.String()), nil, nil) {
+				if m.evictPod(logger, pod, immediateEvictionGracePeriodSeconds, fmt.Sprintf(emptyDirMessageFmt, pod.Spec.Volumes[i].Name, size.String()), nil, nil) {
 					metrics.Evictions.WithLabelValues(signalEmptyDirFsLimit).Inc()
 					return true
 				}
@@ -537,7 +555,7 @@ func (m *managerImpl) emptyDirLimitEviction(podStats statsapi.PodStats, pod *v1.
 	return false
 }
 
-func (m *managerImpl) podEphemeralStorageLimitEviction(podStats statsapi.PodStats, pod *v1.Pod) bool {
+func (m *managerImpl) podEphemeralStorageLimitEviction(logger klog.Logger, podStats statsapi.PodStats, pod *v1.Pod) bool {
 	podLimits := resourcehelper.PodLimits(pod, resourcehelper.PodResourcesOptions{})
 	_, found := podLimits[v1.ResourceEphemeralStorage]
 	if !found {
@@ -553,7 +571,7 @@ func (m *managerImpl) podEphemeralStorageLimitEviction(podStats statsapi.PodStat
 	if podEphemeralStorageTotalUsage.Cmp(podEphemeralStorageLimit) > 0 {
 		// the total usage of pod exceeds the total size limit of containers, evict the pod
 		message := fmt.Sprintf(podEphemeralStorageMessageFmt, podEphemeralStorageLimit.String())
-		if m.evictPod(pod, 0, message, nil, nil) {
+		if m.evictPod(logger, pod, immediateEvictionGracePeriodSeconds, message, nil, nil) {
 			metrics.Evictions.WithLabelValues(signalEphemeralPodFsLimit).Inc()
 			return true
 		}
@@ -562,7 +580,7 @@ func (m *managerImpl) podEphemeralStorageLimitEviction(podStats statsapi.PodStat
 	return false
 }
 
-func (m *managerImpl) containerEphemeralStorageLimitEviction(podStats statsapi.PodStats, pod *v1.Pod) bool {
+func (m *managerImpl) containerEphemeralStorageLimitEviction(logger klog.Logger, podStats statsapi.PodStats, pod *v1.Pod) bool {
 	thresholdsMap := make(map[string]*resource.Quantity)
 	for _, container := range pod.Spec.Containers {
 		ephemeralLimit := container.Resources.Limits.StorageEphemeral()
@@ -579,7 +597,7 @@ func (m *managerImpl) containerEphemeralStorageLimitEviction(podStats statsapi.P
 
 		if ephemeralStorageThreshold, ok := thresholdsMap[containerStat.Name]; ok {
 			if ephemeralStorageThreshold.Cmp(*containerUsed) < 0 {
-				if m.evictPod(pod, 0, fmt.Sprintf(containerEphemeralStorageMessageFmt, containerStat.Name, ephemeralStorageThreshold.String()), nil, nil) {
+				if m.evictPod(logger, pod, immediateEvictionGracePeriodSeconds, fmt.Sprintf(containerEphemeralStorageMessageFmt, containerStat.Name, ephemeralStorageThreshold.String()), nil, nil) {
 					metrics.Evictions.WithLabelValues(signalEphemeralContainerFsLimit).Inc()
 					return true
 				}
@@ -590,30 +608,31 @@ func (m *managerImpl) containerEphemeralStorageLimitEviction(podStats statsapi.P
 	return false
 }
 
-func (m *managerImpl) evictPod(pod *v1.Pod, gracePeriodOverride int64, evictMsg string, annotations map[string]string, condition *v1.PodCondition) bool {
+func (m *managerImpl) evictPod(logger klog.Logger, pod *v1.Pod, gracePeriodOverride int64, evictMsg string, annotations map[string]string, condition *v1.PodCondition) bool {
 	// If the pod is marked as critical and static, and support for critical pod annotations is enabled,
 	// do not evict such pods. Static pods are not re-admitted after evictions.
 	// https://github.com/kubernetes/kubernetes/issues/40573 has more details.
 	if kubelettypes.IsCriticalPod(pod) {
-		klog.ErrorS(nil, "Eviction manager: cannot evict a critical pod", "pod", klog.KObj(pod))
+		logger.Error(nil, "Eviction manager: cannot evict a critical pod", "pod", klog.KObj(pod))
 		return false
 	}
 	// record that we are evicting the pod
 	m.recorder.AnnotatedEventf(pod, annotations, v1.EventTypeWarning, Reason, evictMsg)
 	// this is a blocking call and should only return when the pod and its containers are killed.
-	klog.V(3).InfoS("Evicting pod", "pod", klog.KObj(pod), "podUID", pod.UID, "message", evictMsg)
+	logger.V(3).Info("Evicting pod", "pod", klog.KObj(pod), "podUID", pod.UID, "message", evictMsg)
 	err := m.killPodFunc(pod, true, &gracePeriodOverride, func(status *v1.PodStatus) {
 		status.Phase = v1.PodFailed
 		status.Reason = Reason
 		status.Message = evictMsg
 		if condition != nil {
+			condition.ObservedGeneration = podutil.CalculatePodConditionObservedGeneration(status, pod.Generation, v1.DisruptionTarget)
 			podutil.UpdatePodCondition(status, condition)
 		}
 	})
 	if err != nil {
-		klog.ErrorS(err, "Eviction manager: pod failed to evict", "pod", klog.KObj(pod))
+		logger.Error(err, "Eviction manager: pod failed to evict", "pod", klog.KObj(pod))
 	} else {
-		klog.InfoS("Eviction manager: pod is evicted successfully", "pod", klog.KObj(pod))
+		logger.Info("Eviction manager: pod is evicted successfully", "pod", klog.KObj(pod))
 	}
 	return true
 }
