@@ -27,12 +27,11 @@ import (
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
-	featuregatetesting "k8s.io/component-base/featuregate/testing"
-	"k8s.io/kubernetes/pkg/features"
+	"k8s.io/klog/v2"
 	kubecontainer "k8s.io/kubernetes/pkg/kubelet/container"
 	"k8s.io/kubernetes/pkg/kubelet/prober/results"
 	"k8s.io/kubernetes/pkg/probe"
+	"k8s.io/kubernetes/test/utils/ktesting"
 )
 
 func init() {
@@ -49,6 +48,7 @@ var defaultProbe = &v1.Probe{
 }
 
 func TestAddRemovePods(t *testing.T) {
+	ctx := ktesting.Init(t)
 	noProbePod := v1.Pod{
 		ObjectMeta: metav1.ObjectMeta{
 			UID: "no_probe_pod",
@@ -95,13 +95,13 @@ func TestAddRemovePods(t *testing.T) {
 	}
 
 	// Adding a pod with no probes should be a no-op.
-	m.AddPod(&noProbePod)
+	m.AddPod(ctx, &noProbePod)
 	if err := expectProbes(m, nil); err != nil {
 		t.Error(err)
 	}
 
 	// Adding a pod with probes.
-	m.AddPod(&probePod)
+	m.AddPod(ctx, &probePod)
 	probePaths := []probeKey{
 		{"probe_pod", "readiness", readiness},
 		{"probe_pod", "liveness", liveness},
@@ -136,34 +136,33 @@ func TestAddRemovePods(t *testing.T) {
 func TestAddRemovePodsWithRestartableInitContainer(t *testing.T) {
 	m := newTestManager()
 	defer cleanup(t, m)
-	featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.SidecarContainers, true)
 	if err := expectProbes(m, nil); err != nil {
 		t.Error(err)
 	}
 
 	testCases := []struct {
-		desc                    string
-		probePaths              []probeKey
-		enableSidecarContainers bool
+		desc                        string
+		probePaths                  []probeKey
+		hasRestartableInitContainer bool
 	}{
 		{
-			desc:                    "pod with sidecar (no sidecar containers feature enabled)",
-			probePaths:              nil,
-			enableSidecarContainers: false,
+			desc:                        "pod without sidecar",
+			probePaths:                  nil,
+			hasRestartableInitContainer: false,
 		},
 		{
-			desc: "pod with sidecar (sidecar containers feature enabled)",
+			desc: "pod with sidecar",
 			probePaths: []probeKey{
 				{"restartable_init_container_pod", "restartable-init", liveness},
 				{"restartable_init_container_pod", "restartable-init", readiness},
 				{"restartable_init_container_pod", "restartable-init", startup},
 			},
-			enableSidecarContainers: true,
+			hasRestartableInitContainer: true,
 		},
 	}
 
-	containerRestartPolicy := func(enableSidecarContainers bool) *v1.ContainerRestartPolicy {
-		if !enableSidecarContainers {
+	containerRestartPolicy := func(hasRestartableInitContainer bool) *v1.ContainerRestartPolicy {
+		if !hasRestartableInitContainer {
 			return nil
 		}
 		restartPolicy := v1.ContainerRestartPolicyAlways
@@ -172,8 +171,7 @@ func TestAddRemovePodsWithRestartableInitContainer(t *testing.T) {
 
 	for _, tc := range testCases {
 		t.Run(tc.desc, func(t *testing.T) {
-			featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.SidecarContainers, tc.enableSidecarContainers)
-
+			ctx := ktesting.Init(t)
 			probePod := v1.Pod{
 				ObjectMeta: metav1.ObjectMeta{
 					UID: "restartable_init_container_pod",
@@ -186,7 +184,7 @@ func TestAddRemovePodsWithRestartableInitContainer(t *testing.T) {
 						LivenessProbe:  defaultProbe,
 						ReadinessProbe: defaultProbe,
 						StartupProbe:   defaultProbe,
-						RestartPolicy:  containerRestartPolicy(tc.enableSidecarContainers),
+						RestartPolicy:  containerRestartPolicy(tc.hasRestartableInitContainer),
 					}},
 					Containers: []v1.Container{{
 						Name: "main",
@@ -195,7 +193,7 @@ func TestAddRemovePodsWithRestartableInitContainer(t *testing.T) {
 			}
 
 			// Adding a pod with probes.
-			m.AddPod(&probePod)
+			m.AddPod(ctx, &probePod)
 			if err := expectProbes(m, tc.probePaths); err != nil {
 				t.Error(err)
 			}
@@ -219,6 +217,7 @@ func TestAddRemovePodsWithRestartableInitContainer(t *testing.T) {
 }
 
 func TestCleanupPods(t *testing.T) {
+	ctx := ktesting.Init(t)
 	m := newTestManager()
 	defer cleanup(t, m)
 	podToCleanup := v1.Pod{
@@ -255,8 +254,8 @@ func TestCleanupPods(t *testing.T) {
 			}},
 		},
 	}
-	m.AddPod(&podToCleanup)
-	m.AddPod(&podToKeep)
+	m.AddPod(ctx, &podToCleanup)
+	m.AddPod(ctx, &podToKeep)
 
 	desiredPods := map[types.UID]sets.Empty{}
 	desiredPods[podToKeep.UID] = sets.Empty{}
@@ -281,6 +280,7 @@ func TestCleanupPods(t *testing.T) {
 }
 
 func TestCleanupRepeated(t *testing.T) {
+	ctx := ktesting.Init(t)
 	m := newTestManager()
 	defer cleanup(t, m)
 	podTemplate := v1.Pod{
@@ -298,7 +298,7 @@ func TestCleanupRepeated(t *testing.T) {
 	for i := 0; i < numTestPods; i++ {
 		pod := podTemplate
 		pod.UID = types.UID(strconv.Itoa(i))
-		m.AddPod(&pod)
+		m.AddPod(ctx, &pod)
 	}
 
 	for i := 0; i < 10; i++ {
@@ -307,6 +307,7 @@ func TestCleanupRepeated(t *testing.T) {
 }
 
 func TestUpdatePodStatus(t *testing.T) {
+	ctx := ktesting.Init(t)
 	unprobed := v1.ContainerStatus{
 		Name:        "unprobed_container",
 		ContainerID: "test://unprobed_container_id",
@@ -381,7 +382,7 @@ func TestUpdatePodStatus(t *testing.T) {
 	m.startupManager.Set(kubecontainer.ParseContainerID(startedNoReadiness.ContainerID), results.Success, &v1.Pod{})
 	m.readinessManager.Set(kubecontainer.ParseContainerID(terminated.ContainerID), results.Success, &v1.Pod{})
 
-	m.UpdatePodStatus(&v1.Pod{
+	m.UpdatePodStatus(ctx, &v1.Pod{
 		ObjectMeta: metav1.ObjectMeta{
 			UID: testPodUID,
 		},
@@ -453,10 +454,10 @@ func TestUpdatePodStatusWithInitContainers(t *testing.T) {
 	m.startupManager.Set(kubecontainer.ParseContainerID(started.ContainerID), results.Success, &v1.Pod{})
 
 	testCases := []struct {
-		desc                    string
-		expectedStartup         map[probeKey]bool
-		expectedReadiness       map[probeKey]bool
-		enableSidecarContainers bool
+		desc                        string
+		expectedStartup             map[probeKey]bool
+		expectedReadiness           map[probeKey]bool
+		hasRestartableInitContainer bool
 	}{
 		{
 			desc: "init containers",
@@ -470,10 +471,10 @@ func TestUpdatePodStatusWithInitContainers(t *testing.T) {
 				{testPodUID, started.Name, readiness}:    false,
 				{testPodUID, terminated.Name, readiness}: true,
 			},
-			enableSidecarContainers: false,
+			hasRestartableInitContainer: false,
 		},
 		{
-			desc: "init container with SidecarContainers feature",
+			desc: "init container with Always restartPolicy",
 			expectedStartup: map[probeKey]bool{
 				{testPodUID, notStarted.Name, startup}: false,
 				{testPodUID, started.Name, startup}:    true,
@@ -484,7 +485,7 @@ func TestUpdatePodStatusWithInitContainers(t *testing.T) {
 				{testPodUID, started.Name, readiness}:    true,
 				{testPodUID, terminated.Name, readiness}: false,
 			},
-			enableSidecarContainers: true,
+			hasRestartableInitContainer: true,
 		},
 	}
 
@@ -498,7 +499,7 @@ func TestUpdatePodStatusWithInitContainers(t *testing.T) {
 
 	for _, tc := range testCases {
 		t.Run(tc.desc, func(t *testing.T) {
-			featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.SidecarContainers, tc.enableSidecarContainers)
+			ctx := ktesting.Init(t)
 			podStatus := v1.PodStatus{
 				Phase: v1.PodRunning,
 				InitContainerStatuses: []v1.ContainerStatus{
@@ -506,7 +507,7 @@ func TestUpdatePodStatusWithInitContainers(t *testing.T) {
 				},
 			}
 
-			m.UpdatePodStatus(&v1.Pod{
+			m.UpdatePodStatus(ctx, &v1.Pod{
 				ObjectMeta: metav1.ObjectMeta{
 					UID: testPodUID,
 				},
@@ -514,15 +515,15 @@ func TestUpdatePodStatusWithInitContainers(t *testing.T) {
 					InitContainers: []v1.Container{
 						{
 							Name:          notStarted.Name,
-							RestartPolicy: containerRestartPolicy(tc.enableSidecarContainers),
+							RestartPolicy: containerRestartPolicy(tc.hasRestartableInitContainer),
 						},
 						{
 							Name:          started.Name,
-							RestartPolicy: containerRestartPolicy(tc.enableSidecarContainers),
+							RestartPolicy: containerRestartPolicy(tc.hasRestartableInitContainer),
 						},
 						{
 							Name:          terminated.Name,
-							RestartPolicy: containerRestartPolicy(tc.enableSidecarContainers),
+							RestartPolicy: containerRestartPolicy(tc.hasRestartableInitContainer),
 						},
 					},
 				},
@@ -554,14 +555,15 @@ func TestUpdatePodStatusWithInitContainers(t *testing.T) {
 	}
 }
 
-func (m *manager) extractedReadinessHandling() {
+func (m *manager) extractedReadinessHandling(logger klog.Logger) {
 	update := <-m.readinessManager.Updates()
 	// This code corresponds to an extract from kubelet.syncLoopIteration()
 	ready := update.Result == results.Success
-	m.statusManager.SetContainerReadiness(update.PodUID, update.ContainerID, ready)
+	m.statusManager.SetContainerReadiness(logger, update.PodUID, update.ContainerID, ready)
 }
 
 func TestUpdateReadiness(t *testing.T) {
+	logger, ctx := ktesting.NewTestContext(t)
 	testPod := getTestPod()
 	setTestProbe(testPod, readiness, v1.Probe{})
 	m := newTestManager()
@@ -569,7 +571,7 @@ func TestUpdateReadiness(t *testing.T) {
 
 	// Start syncing readiness without leaking goroutine.
 	stopCh := make(chan struct{})
-	go wait.Until(m.extractedReadinessHandling, 0, stopCh)
+	go wait.Until(func() { m.extractedReadinessHandling(logger) }, 0, stopCh)
 	defer func() {
 		close(stopCh)
 		// Send an update to exit extractedReadinessHandling()
@@ -580,9 +582,9 @@ func TestUpdateReadiness(t *testing.T) {
 	exec.set(probe.Success, nil)
 	m.prober.exec = &exec
 
-	m.statusManager.SetPodStatus(testPod, getTestRunningStatus())
+	m.statusManager.SetPodStatus(logger, testPod, getTestRunningStatus())
 
-	m.AddPod(testPod)
+	m.AddPod(ctx, testPod)
 	probePaths := []probeKey{{testPodUID, testContainerName, readiness}}
 	if err := expectProbes(m, probePaths); err != nil {
 		t.Error(err)
