@@ -21,20 +21,25 @@ import (
 
 	genericapiserver "k8s.io/apiserver/pkg/server"
 	"k8s.io/kubernetes/pkg/controlplane"
+	controlplaneapiserver "k8s.io/kubernetes/pkg/controlplane/apiserver"
 )
 
 // This test references methods that OpenShift uses to customize the apiserver on startup, that
 // are not referenced directly by an instance.
 func TestApiserverExportsSymbols(t *testing.T) {
 	_ = &controlplane.Config{
-		GenericConfig: &genericapiserver.Config{
-			EnableMetrics: true,
-		},
-		ExtraConfig: controlplane.ExtraConfig{
-			EnableLogsSupport: false,
+		ControlPlane: controlplaneapiserver.Config{
+			Generic: &genericapiserver.Config{
+				EnableMetrics: true,
+			},
+			Extra: controlplaneapiserver.Extra{
+				EnableLogsSupport: false,
+			},
 		},
 	}
 	_ = &controlplane.Instance{
-		GenericAPIServer: &genericapiserver.GenericAPIServer{},
+		ControlPlane: &controlplaneapiserver.Server{
+			GenericAPIServer: &genericapiserver.GenericAPIServer{},
+		},
 	}
 }
